      SUBROUTINE TIME6T(TITLE,NPOL,TMIN,TMAX,TAU,ITORDR,IPRINT,IPRSTP,
     1                  IPRITT,NMESH,RMESH,IBOUND,FNOUT,IOUT,FMATR,
     2                  IOUM,EVWFN,IOUF,TOT,ITOT,ZTOT,MTOT,MITOT,MZTOT)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                      C
C                                                                      C
C             ####### # #     # ####### ####### #######                C
C                #    # ##   ## #       #          #                   C
C                #    # # # # # #       #          #                   C
C                #    # #  #  # #####   #######    #                   C
C                #    # #     # #       #     #    #                   C
C                #    # #     # #       #     #    #                   C
C                #    # #     # ####### #######    #                   C
C                                                                      C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                      C
C                                                                      C
C                    version  1.0  from  20.10.2008                    C
C                                                                      C
C                                                                      C
C                              written  by                             C
C                                                                      C
C                                                                      C
C              S.I. Vinitsky, A.A. Gusev,  O. Chuluunbaatar            C
C                                                                      C
C                   Joint Institute for Nuclear Research,              C
C                    Dubna, 141980 Moscow region, Russia               C
C                                                                      C
C----------------------------------------------------------------------C
C                                                                      C
C                                                                      C
C  Program TIME6T calculates solution of the one dimensional           C
C  time-dependent Schrodinger equation using the decomposition         C
C  of the evolution operator via the explicit Magnus expansion         C
C  up to the sixth order of accuracy with respect to the time          C
C  step. The all needed matrix elements are solved using               C
C  high-order accuracy approximations of the finite element            C
C  method.                                                             C
C                                                                      C
C----------------------------------------------------------------------C
C                                                                      C
C                                                                      C
C                        I N P U T  D A T A                            C
C                        ------------------                            C
C                                                                      C
C TITLE   CHARACTER  title of the run to be printed on the output      C
C                    listing. The title should be no longer than       C
C                    70 characters.                                    C
C NPOL    INTEGER    order of finite-element shape functions           C
C                    (interpolating Lagrange polynomials).             C
C                    Usually set to 6.                                 C
C TMIN    REAL*8     begin of time interval.                           C                            
C TMAX    REAL*8     end of time interval.                             C                            
C TAU     REAL*8     time step.                                        C 
C ITORDR  INTEGER    order of Magnus expansion, 1 , 2 , 3.             C
C IPRINT  INTEGER    level of print:                                   C
C                    = 0 -- minimal level of print. The initial data,  C
C                    short information about the numerical scheme      C
C                    parameters, main flags and keys, and energy       C
C                    values calculated are printed out;                C
C                    = 1 -- solutions calculated are printed           C
C                    out with step IPRSTP additionally;                C
C                    = 2 -- the highest level of print.                C
C IPRSTP  INTEGER    step of spatial variable x with which solutions   C
C                    are printed out.                                  C
C IPRITT  INTEGER    step of time variable t with which solutions      C
C                    are printed out.                                  C
C NMESH   INTEGER    dimension of array RMESH. NMESH always should be  C
C                    odd and > = 3.                                    C
C RMESH   REAL*8     array RMESH contains information about            C
C                    subdivision of interval [x_min,x_max] of          C
C                    spatial variable x on subintervals. The whole     C
C                    interval [x_min,x_max] is divided as follows:     C
C                    RMESH(1) = x_min, RMESH(NMESH) = x_max, and the   C
C                    values of RMESH(I) set the number of elements     C
C                    for each subinterval [RMESH(I-1), RMESH(I+1)],    C
C                    where I=2, 4, ... , NMESH-1.                      C
C IBOUND  INTEGER    parameter defining the type of boundary           C
C                    conditions set in the boundary points x = x_min   C
C                    and x = x_max:                                C
C                    = 1 -- the Dirichlet -- Dirichlet boundary        C
C                    conditions;                                       C
C                    = 2 -- the Dirichlet -- Neumann boundary          C
C                    conditions;                                       C
C                    = 3 -- the Neumann -- Dirichlet boundary          C
C                    conditions;                                       C
C                    =  4 -- the Neumann -- Neumann boundary           C
C                    conditions.                                       C
C FNOUT   CHARACTER  name of the output file (up to 55 characters)     C
C                    for printing out the results of the calculation.  C
C                    It is system specific and may include a complete  C
C                    path to the file location.                        C
C IOUT    INTEGER    number of the output logical device for printing  C
C                    out the results of the calculation (usually set   C
C                    to 7).                                            C
C FMATR   CHARACTER  name of the scratch file (up to 55 characters)    C
C                    for storing matrices.                             C
C IOUM    INTEGER    number of the logical device for storing          C
C                    matrices.                                         C
C EVWFN   CHARACTER  name of the output file (up to 55 characters) for C
C                    storing the results of the calculation, namely,   C
C                    finite-element grid points, and solutions.        C
C                    It is used only if IOUF > 0.                      C
C IOUF    INTEGER    number of the logical device for storing data     C
C                    into file EVWFN.                                  C
C TOT     REAL*8     working vector of the DOUBLE PRECISION type.      C
C ITOT    INTEGER    working vector of the INTEGER type.               C
C ZTOT    COMPLEX*16 working vector of the DOUBLE COMPLEX type.        C
C MTOT    INTEGER    dimension of the DOUBLE PRECISION working array   C
C                    TOT. The last address ILAST of array TOT is       C
C                    calculated and then compared with the given value C
C                    of MTOT. If ILAST > MTOT the message about an     C
C                    error is printed and the execution of the program C
C                    is aborted. In the last case, in order to carry   C
C                    out the required calculation it is necessary to   C
C                    increase the dimension MTOT of array TOT to the   C
C                    quantity ILAST taken from the message.            C
C MITOT   INTEGER    dimension of the INTEGER working array ITOT. The  C
C                    last address ILAST of array ITOT is calculated    C
C                    and then compared with the given value of MITOT.  C
C                    If ILAST > MITOT the message about an error is    C
C                    printed and the execution of the program is       C
C                    aborted. In the last case, in order to carry out  C
C                    the required calculation it is necessary to       C
C                    increase the dimension MITOT of array ITOT to the C
C                    quantity ILAST taken from the message.            C
C MZTOT   INTEGER    dimension of the DOUBLE COMPLEX working array     C
C                    ZTOT. The last address ILAST of array ZTOT is     C
C                    calculated and then compared with the given value C
C                    of MZTOT. If ILAST > MZTOT the message about an   C
C                    error is printed and the execution of the program C
C                    is aborted. In the last case, in order to carry   C
C                    out the required calculation it is necessary to   C
C                    increase the dimension MZTOT of array ZTOT to the C
C                    quantity ILAST taken from the message.            C
C                                                                      C
C----------------------------------------------------------------------C
C                                                                      C
C                                                                      C
C                        O U T P U T  D A T A                          C
C                        --------------------                          C
C                                                                      C
C The result of the calculation of solutions are written using         C
C unformatted segmented records into file EVWFN, according to          C
C the following operator:                                              C
C                                                                      C
C            WRITE(IOUF) NN,NGRID,TT,TAU,(XGRID(J),J=1,NGRID)          C
C    1                  ,(ZU(J),J=1,NN)                                C
C                                                                      C
C In the above, parameters presented in the WRITE statement have the   C
C following meaning:                                                   C
C NGRID is the number of finite-element grid points.                   C
C NN    is the number of finite-element grid points of solution.       C
C TT    is the value of time variable.                                 C 
C TAU   is the time step.                                              C 
C Array XGRID contains the values of the finite-element                C
C       grid points.                                                   C
C Array ZU contains calculated solution.                               C
C                                                                      C
C----------------------------------------------------------------------C
C                                                                      C
C                                                                      C
C                         PUBLISHED PROGRAMS USED                      C
C                         -----------------------                      C
C                                                                      C
C          Program GAULEG to calculate nodes and weights of the        C
C          Gauss-Legendre quadrature. Published in: W.H. Press,        C
C          B.F. Flanery, S.A. Teukolsky, and W.T. Vetterley,           C
C          Numerical Recipes: The Art of Scientific Computing          C
C          (Cambridge University Press, Cambridge, 1986)               C
C                                                                      C
C----------------------------------------------------------------------C
C                                                                      C
      IMPLICIT REAL*8 (A-H,O-Y)
      IMPLICIT COMPLEX*16 (Z)
      DIMENSION TOT(MTOT),ITOT(MITOT),RMESH(NMESH),ZTOT(MZTOT)
      CHARACTER*70 TITLE
      CHARACTER*55 FNOUT,FMATR,EVWFN
      COMMON /PARAM/ NN,NGRID,NWK,NNM 
      DATA DZERO / 0.D0 /, ONE / 1.D0 /
C
C      *******************************************
C      ***                                     ***
C      ***    I N P U T  D A T A  P H A S E    ***
C      ***                                     ***
C      *******************************************
C
C      THE FOLLOWING FILES ARE USED
C
C          IOUT   =  OUTPUT LOGICAL DEVICE NUMBER FOR
C                    PRINTING OUT RESULTS OF THE CALCULATION
C          IOUM   =  OUTPUT MATRICES
C          IOUF   =  OUTPUT LOGICAL DEVICE FOR STORING
C                    WAVE FUNCTIONS
C
      OPEN (UNIT=IOUT,STATUS='UNKNOWN',FILE=FNOUT)
      OPEN (UNIT=IOUM,STATUS='UNKNOWN',FILE=FMATR,FORM='UNFORMATTED')
      IF (IOUF .GT. 0)
     1OPEN (UNIT=IOUF,STATUS='UNKNOWN',FILE=EVWFN,FORM='UNFORMATTED')
C
C      THE RELATIVE ERROR OF THE CALCULATION
C
      EPSY  = ONE
   11 EPSY  = EPSY / 2
      ONEP = ONE + EPSY
      IF (ONEP .GT. ONE) GO TO 11
      EPSY  = 2 * EPSY
C
      DO I = 1 , MTOT
        TOT(I) = DZERO
      END DO
      DO I = 1 , MITOT
        ITOT(I) = 0
      END DO
      DO I = 1 , MZTOT
        ZTOT(I) = DZERO
      END DO
C
C      FORM AND CHECK INPUT DATA
C
      MDIM = 1
      NNODE  = NPOL + 1
      NNODES = NNODE * MDIM
      NGQ    = NNODE
      NSTF   = NNODES * (NNODES + 1) / 2
C
C      SET GAUSSIAN QUADRATURE NODES AND WEIGHTS
C
      IXG = 1
      IWG = IXG + NGQ
C
      CALL GAULEG(-ONE,ONE,TOT(IXG),TOT(IWG),NGQ,EPSY)
C
C      *****************************************************
C      ***                                               ***
C      ***   N O D A L  G E N E R A T I N G  P H A S E   ***
C      ***                                               ***
C      *****************************************************
C
      IGRID = IWG + NGQ
      NELEM = 0
C
      CALL FEGRID(RMESH,NMESH,TOT(IGRID),NGRID,NELEM,NPOL)
C
      IF (IPRINT .GE. 0) THEN
        WRITE(IOUT,1000) TITLE
        WRITE(IOUT,1020) NELEM,NGRID,NPOL,NGQ,IBOUND
        WRITE(IOUT,1030)
        NO    = 0
        DO 20 I = 3 , NMESH , 2
           NO  = NO + 1
           NM  = IDINT(RMESH(I-1))
           XM  = (RMESH(I) - RMESH(I-2)) / NM
           XS  = XM / NPOL
           WRITE(IOUT,1040) NO,NM,RMESH(I-2),XM,XS,RMESH(I)
   20   CONTINUE
      END IF
C
      INODE  = 1
      INODES = INODE   +  NNODE  *  NELEM
C
C      GENERATE AND STORE NODAL DATA
C
      CALL NODGEN(ITOT(INODE),NNODE,ITOT(INODES),NNODES,MDIM,NELEM,
     1            IPRINT,IOUT)
C
      NN    = MDIM * NGRID
      NNE   = NN
      NNM   = NN + 1
C
      IMAXA = INODES +  NNODES * NELEM
      IMHT  = IMAXA  +  NNM
      INHT  = IMHT   +  NNM
      ILAST = INHT   +  NNM
C
      IF (IPRINT .GE. 2) WRITE(IOUT,1070) ILAST
C
      IF (ILAST .GT. MITOT) THEN
         CALL ERRDIM(ILAST - MITOT,1,IOUT)
         WRITE(IOUT,1070) ILAST
      END IF
C
C      SET THE DIRICHLET AND/OR NEUMANN BOUNDARY CONDITIONS
C
      CALL BOUNDC(ITOT(INODE),ITOT(INODES),ITOT(IMAXA),ITOT(IMHT),
     1            ITOT(INHT),IBOUND,NNODE,NNODES,NELEM,MDIM,NN,NNE,
     2            IPRINT,IOUT)
C
C      CALCULATE ADDRESSES OF DIAGONAL ELEMENTS
C      IN BANDED STIFFNESS AND MASS MATRICES
C
      CALL MAXHT(ITOT(IMAXA),ITOT(IMHT),ITOT(INODES),NNODES,NELEM,
     1           NN,NNM,NWK,NWM,MK,IPRINT,IOUT)
C
      MMK  = NWK / NN
      IF (IPRINT .GE. 0) WRITE(IOUT,1050) NN,NWK,MK,MMK
C
C      **********************************************************
C      ***                                                    ***
C      ***    E L E M E N T  G E N E R A T I N G  P H A S E   ***
C      ***                                                    ***
C      **********************************************************
C
C      CALCULATE AND ASSEMBLE STIFFNESS AND MASS
C      ELEMENT MATRICES INTO THE GLOBAL ONES
C
      IPSI   = IGRID  +  NGRID
      IDPSI  = IPSI   +  NGQ    * NNODE
      IA     = IDPSI  +  NGQ    * NNODE
      IB     = IA     +  NWK
      IESTFA = IB     +  NWK
      IESTFB = IESTFA +  NSTF
      IESTF  = IESTFB +  NSTF
      IEMSS  = IESTF  +  NNODES * NNODES
      IEST1  = IEMSS  +  NNODES * NNODES
      IEMS1  = IEST1  +  NNODE  * NNODE
      IXNODE = IEMS1  +  NNODE  * NNODE
      IUNODE = IXNODE +  NNODE
      IQNODE = IUNODE +  NNODE
      ILAST  = IQNODE +  NNODE
C
      IF (IPRINT .GE. 2) WRITE(IOUT,1080) ILAST
C
      IF (ILAST .GT. MTOT) THEN
         CALL ERRDIM(ILAST - MTOT,2,IOUT)
         WRITE(IOUT,1080) ILAST
      END IF
C
C      CALCULATE AND ASSEMBLE MASS MATRIX
C
      CALL ASSMBM(ITOT(IMAXA),TOT(IB),TOT(IEMSS),TOT(IESTFB),TOT(IEMS1),
     1            TOT(IGRID),TOT(IXNODE),TOT(IUNODE),TOT(IQNODE),
     2            TOT(IPSI),TOT(IXG),TOT(IWG),ITOT(INODE),ITOT(INODES),
     3            NELEM,NNODE,NNODES,NGQ,NNM,NWK,NGRID,NSTF,IOUT)
C . . . . . . . . . . . . . . . . . . . . . . . . . .
C
C           *********************************************
C           ***                                       ***
C           ***       TIME DEPENDENT PROBLEM          ***
C           ***                                       ***
C           *********************************************
C
      IZU    = 1
      IZT    = IZU    +   NN
      IZM    = IZT    +   NN
      ILAST  = IZM    +   NWK 
C
      IF (IPRINT .GE. 2) WRITE(IOUT,1090) ILAST
C
      IF (ILAST .GT. MZTOT) THEN
         CALL ERRDIM(ILAST - MZTOT,3,IOUT)
         WRITE(IOUT,1090) ILAST
      END IF
C
      CALL TMSOLV(ITOT(IMAXA),TOT(IESTF),TOT(IEMSS),TOT(IESTFA),
     1            TOT(IESTFB),TOT(IEST1),TOT(IEMS1),TOT(IXNODE),
     2            TOT(IUNODE),TOT(IQNODE),TOT(IPSI),TOT(IDPSI),TOT(IXG),
     3            TOT(IWG),ZTOT(IZU),ZTOT(IZT),ZTOT(IZM),TOT(IA),
     4            TOT(IB),TOT(IGRID),ITOT(INODE),ITOT(INODES),NGQ,NNODE,
     5            NNODES,NGRID,NN,NNM,NWK,NWM,NELEM,NSTF,TMIN,TMAX,TAU,
     6            ITORDR,IPRINT,IPRSTP,IPRITT,IOUT,IOUM,IOUF,IBOUND)
C
      CLOSE (UNIT=IOUM)
      RETURN
 1000 FORMAT(/1X,'PROBLEM: ',A70/1X,8('*')/)
 1020 FORMAT(/15X,'C O N T R O L  I N F O R M A T I O N'/
     *        15X,'------------------------------------'//5X,
     * 'NUMBER OF FINITE ELEMENTS . . . . . . . . (NELEM ) = ',I6/5X,
     * 'NUMBER OF GRID POINTS . . . . . . . . . . (NGRID ) = ',I6/5X,
     * 'ORDER OF SHAPE FUNCTIONS. . . . . . . . . (NPOL  ) = ',I6/5X,
     * 'ORDER OF GAUSS-LEGENDRE QUADRATURE. . . . (NGQ   ) = ',I6/5X,
     * 'BOUNDARY CONDITION CODE . . . . . . . . . (IBOUND) = ',I6/5X)
 1030 FORMAT(/4X,
     * 'SUBDIVISION OF RHO-REGION ON THE FINITE-ELEMENT GROUPS:'/4X,
     * '****************************************************** ',//
     *2X,' NO OF  NUMBER OF  BEGIN OF  LENGTH OF    GRID     END OF  '/
     *2X,' GROUP  ELEMENTS   INTERVAL   ELEMENT     STEP    INTERVAL'/
     *2X,' -----  --------   --------   -------     ----    --------')
 1040 FORMAT(2X,I4,4X,I6,4X,F9.5,1X,F9.5,1X,F10.6,1X,F10.3)
 1050 FORMAT(//15X,'T O T A L  S Y S T E M  D A T A  '/
     *         15X,'-------------------------------  '//5X,
     * 'TOTAL NUMBER OF ALGEBRAIC EQUATIONS. . . . (NN ) =',I9/5X,
     * 'TOTAL NUMBER OF MATRIX ELEMENTS. . . . . . (NWK) =',I9/5X,
     * 'MAXIMUM HALF BANDWIDTH . . . . . . . . . . (MK ) =',I9/5X,
     * 'MEAN    HALF BANDWIDTH . . . . . . . . . . (MMK) =',I9/)
 1060 FORMAT(/80('*')/)
 1070 FORMAT(/5X,'LAST ADDRESS OF ARRAY ITOT USED = ',I10/)
 1080 FORMAT(/5X,'LAST ADDRESS OF ARRAY TOT USED = ',I10/)
 1090 FORMAT(/5X,'LAST ADDRESS OF ARRAY ZTOT USED = ',I10/)
 1100 FORMAT(/5X,'TYPE OF CONSIDERING PROBLEM',5X,A70/)
      END
C
      SUBROUTINE TMSOLV(MAXA,ESTF,EMSS,ESTFA,ESTFB,EST1,EMS1,XNODE,
     1                  UNODE,QNODE,PSI,DPSI,XG,WG,ZU,ZT,ZM,A,B,XGRID,
     2                  NODE,NODES,NGQ,NNODE,NNODES,NGRID,NN,NNM,NWK,
     3                  NWM,NELEM,NSTF,TMIN,TMAX,TAU,ITORDR,
     4                  IPRINT,IPRSTP,IPRITT,IOUT,IOUM,IOUF,IBOUND)
C . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
C .                                                                   .
C .   P R O G R A M                                                   .
C .                  TO CALCULATE THE SOLUTIONS, PRINT OUT            .
C .                  THE SOLUTIONS CALCULATED INTO THE IOUF           .
C .                  FILE, IF NECESSARY                               .
C .                                                                   .
C . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
      IMPLICIT REAL*8 (A-H,O-Y)
      IMPLICIT COMPLEX*16 (Z) 
      DIMENSION MAXA(NNM),NODE(NNODE,NELEM),NODES(NNODES,NELEM),A(NWK),
     1          B(NWK),XGRID(NGRID),ESTF(NNODES,NNODES),
     2          EMSS(NNODES,NNODES),ESTFA(NSTF),ESTFB(NSTF),
     3          EST1(NNODE,NNODE),EMS1(NNODE,NNODE),XNODE(NNODE),
     4          UNODE(NNODE),QNODE(NNODE),PSI(NNODE,NGQ),
     5          DPSI(NNODE,NGQ),XG(NGQ),WG(NGQ),ZU(NN),ZT(NN),ZM(NWK),
     6          ZALP(3,3)
      DATA DZERO / 0.D0 /
C
      ZI = (0.D0,1.D0)
      PI = DACOS(-1.D0)
      REWIND IOUM
      WRITE(IOUM) (B(I),I=1,NWK)  ! MASS MATRIX
C
      ZALP(1,1) = (0.D0,-0.5D0)
      ZALP(2,1) = (-0.1443375672974064D0,-0.25D0)
      ZALP(2,2) = ( 0.1443375672974064D0,-0.25D0)
      ZALP(3,1) = (-0.1357999257081538D0,-0.1423427884419439D0)
      ZALP(3,2) = ( 0.D0                ,-0.2153144231161122D0) 
      ZALP(3,3) = ( 0.1357999257081538D0,-0.1423427884419439D0)
C  
      NTMAX = (TMAX - TMIN) / TAU
      ITT = 0
      T = TMIN - TAU
C
      KEY=1
      IF (IBOUND .GE. 3) KEY=0
C
      CALL DINIT(KEY,NN,NGRID,XGRID,ZU)
      CALL MULTC(ZT,B,ZU,MAXA,NN,NWM)
      ZSUM = DZERO 
      DO I = 1 , NN
        ZSUM = ZSUM + DCONJG(ZU(I)) * ZT(I)
      END DO
      DNORM = CDSQRT(ZSUM)
      DO I = 1 , NN
        ZU(I) = ZU(I) / DNORM 
      END DO
C
      DO IT = 1 , NTMAX
C
        T = T + TAU
        TT = T + TAU / 2
        ITT = ITT + 1
C
C      CALCULATE AND ASSEMBLE AA AND SS MATRICES
C
        CALL ASSMBS(MAXA,A,B,ESTF,EMSS,ESTFA,ESTFB,EST1,EMS1,XGRID,
     1              XNODE,UNODE,QNODE,PSI,DPSI,XG,WG,NODE,NODES,
     2              NELEM,NNODE,NNODES,NGQ,NNM,NWK,NGRID,NSTF,
     3              IOUT,TT,TAU,ITORDR)
C
        REWIND IOUM 
        READ (IOUM)                 ! MASS MATRIX 
        WRITE(IOUM) (A(J),J=1,NWK)  ! STIF MATRIX
        WRITE(IOUM) (B(J),J=1,NWK)  ! S MATRIX
C
        IF (ITORDR .NE. 1) THEN
C
          REWIND IOUM 
          READ (IOUM) (B(J),J=1,NWK)  ! MASS MATRIX
          READ (IOUM) 
          READ (IOUM) (A(J),J=1,NWK)  ! S MATRIX
C
          NTORDR = ITORDR / 2
C  
          DO K = 1 , NTORDR
C
            DO J = 1 , NWK
              ZM(J) = B(J) - ZALP(NTORDR,K) * A(J)
            END DO
            CALL MULTCC(ZT,ZM,ZU,MAXA,NN,NWM)
            DO J = 1 , NWK
              ZM(J) = B(J) - DCONJG(ZALP(NTORDR,K)) * A(J)
            END DO
            ISH = 1
            CALL DECOMC(ZM,MAXA,NN,ISH,IFPR,IOUT)
            CALL REDBAC(ZM,ZT,MAXA,NN)
            DO J = 1 , NN
              ZU(J) = ZT(J)
            END DO
C
          END DO
C
        END IF
C
        REWIND IOUM 
        READ (IOUM) (B(J),J=1,NWK)  ! MASS MATRIX
        READ (IOUM) (A(J),J=1,NWK)  ! STIF MATRIX
C                             
        DO K = 1 , ITORDR
C
          DO J = 1 , NWK
            ZM(J) = B(J) + TAU * ZALP(ITORDR,K) * A(J)
          END DO
          CALL MULTCC(ZT,ZM,ZU,MAXA,NN,NWM)
          DO J = 1 , NWK
            ZM(J) = B(J) + TAU * DCONJG(ZALP(ITORDR,K)) * A(J)
          END DO
          ISH = 1
          CALL DECOMC(ZM,MAXA,NN,ISH,IFPR,IOUT)
          CALL REDBAC(ZM,ZT,MAXA,NN)
          DO J = 1 , NN
            ZU(J) = ZT(J)
          END DO
C
        END DO
C
        IF (ITORDR .NE. 1) THEN
C
          REWIND IOUM 
          READ (IOUM) (B(J),J=1,NWK)  ! MASS MATRIX
          READ (IOUM) 
          READ (IOUM) (A(J),J=1,NWK)  ! S MATRIX
C
          NTORDR = ITORDR / 2
C  
          DO K = 1 , NTORDR
C
            DO J = 1 , NWK
              ZM(J) = B(J) + ZALP(NTORDR,K) * A(J)
            END DO
            CALL MULTCC(ZT,ZM,ZU,MAXA,NN,NWM)
            DO J = 1 , NWK
              ZM(J) = B(J) + DCONJG(ZALP(NTORDR,K)) * A(J)
            END DO
            ISH = 1
            CALL DECOMC(ZM,MAXA,NN,ISH,IFPR,IOUT)
            CALL REDBAC(ZM,ZT,MAXA,NN)
            DO J = 1 , NN
              ZU(J) = ZT(J)
            END DO
C
          END DO
C
        END IF
C
        TT = TT + TAU / 2
C
        IF (ITT .EQ. IPRITT) THEN
C
          REWIND IOUM 
          READ (IOUM) (B(J),J=1,NWK)  ! MASS MATRIX
          ITT = 0
          CALL MULTC(ZT,B,ZU,MAXA,NN,NWM)
          ZSUM = DZERO 
          DO J = 1 , NN
            ZSUM = ZSUM + DCONJG(ZU(J)) * ZT(J)
          END DO
          WRITE(*   ,1040) TT,DREAL(ZSUM)
          WRITE(IOUT,1040) TT,DREAL(ZSUM)
          IF (IOUF .NE. 0) THEN 
             WRITE(IOUF) NN,NGRID,TT,TAU,(XGRID(J),J=1,NGRID)
     1                  ,(ZU(J),J=1,NN) 
          END IF
C
C      PRINT THE SOLUTIONS
C
          IF (IPRINT .GE. 1) THEN
C
            WRITE(IOUT,1030)
            DO 45 II = 1 , NN , IPRSTP
               WRITE(IOUT,1020) XGRID(II+KEY),ZU(II)
   45       CONTINUE
            WRITE(IOUT,1010)
C
          END IF
C
        END IF
C
      END DO
C
      REWIND IOUM
      READ (IOUM) (B(I),I=1,NWK)
      REWIND IOUM
      WRITE(IOUM) NWK, (B(I),I=1,NWK),NNM, (MAXA(I),I=1,NNM)
      RETURN
 1010 FORMAT(/80('*')/)
 1020 FORMAT(1X,F10.4,1X,2(1X,D10.4))
 1030 FORMAT(//8X,'R',8X,'   S O L U T I O N ',
     *        /8X,'-',8X,'-------------------'/)
 1040 FORMAT(1X,'TT=',F12.6,3X,'NORM=',F12.6) 
      END
C
      SUBROUTINE ASSMBS(MAXA,A,B,ESTF,EMSS,ESTFA,ESTFB,EST1,EMS1,XGRID,
     1                  XNODE,UNODE,QNODE,PSI,DPSI,XG,WG,NODE,NODES,
     2                  NELEM,NNODE,NNODES,NGQ,NNM,NWK,NGRID,NSTF,
     3                  IOUT,TT,TAU,ITORDR)
C . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
C .                                                                   .
C .   P R O G R A M                                                   .
C .                  TO CALCULATE AND ASSEMBLE THE AA AND SS          .
C .                  ELEMENT MATRICES INTO THE GLOBAL                 .
C .                  AA AND SS MATRICES                               .
C .                                                                   .
C . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION MAXA(NNM),A(NWK),B(NWK),ESTF(NNODES,NNODES),
     1          EMSS(NNODES,NNODES),ESTFA(NSTF),ESTFB(NSTF),
     2          EST1(NNODE,NNODE),EMS1(NNODE,NNODE),XGRID(NGRID),
     3          XNODE(NNODE),UNODE(NNODE),QNODE(NNODE),PSI(NNODE,NGQ),
     4          DPSI(NNODE,NGQ),XG(NGQ),WG(NGQ),NODE(NNODE,NELEM),
     5          NODES(NNODES,NELEM)
      DATA DZERO / 0.D0 /
C
      DO 15 IG = 1 , NGQ
C
          CALL SHAPEF(XG(IG),UNODE,QNODE,XNODE,NNODE)
C
          DO 14 IN = 1 , NNODE
             PSI (IN,IG) = UNODE(IN)
             DPSI(IN,IG) = QNODE(IN)
   14     CONTINUE
C
   15 CONTINUE
C
C      BEGIN BIG LOOP ON ELEMENTS
C
C      CALCULATE AA AND SS MATRICES
C
C      INITIALIZE ARRAYS A(NWK), B(NWK)
C
      DO 10 I = 1 , NWK
         A(I) = DZERO
         B(I) = DZERO
   10 CONTINUE
C
      DO 220 NEL = 1 , NELEM
C
         DO 40 II = 1 , NNODES
         DO 40 JJ = 1 , NNODES
            ESTF(II,JJ) = DZERO
            EMSS(II,JJ) = DZERO
   40    CONTINUE
C
         DO 50 LL = 1 , NNODE
            NE           = NODE(LL,NEL)
            XNODE(LL)    = XGRID(NE)
   50    CONTINUE
C
C      CALCULATE ELEMENT MATRICES FOR DIAGONAL
C      ELEMENTS OF AA AND SS MATRICES
C
         CALL ESTIFD(XNODE,XG,WG,NGQ,NNODE,EST1,EMS1,PSI,DPSI,
     1               TT,TAU,ITORDR)
C
            DO 70 NI = 1  , NNODE
            DO 70 NJ = NI , NNODE
C
               ESTF(NI,NJ) = ESTF(NI,NJ) + EST1(NI,NJ)
               EMSS(NI,NJ) = EMSS(NI,NJ) + EMS1(NI,NJ)
C
   70       CONTINUE
C
         NL  = 0
         DO 210 IL = 1  , NNODES
         DO 210 JL = IL , NNODES
            NL        = NL + 1
            ESTFA(NL) = ESTF(IL,JL)
            ESTFB(NL) = EMSS(IL,JL)
C
  210    CONTINUE
C
C      ASSEMBLE ELEMENT MATRICES INTO GLOBAL
C      AA AND SS MATRICES
C
         CALL ADDVEC(A,NWK,MAXA,NNM,ESTFA,NSTF,NODES(1,NEL),NNODES,1)
         CALL ADDVEC(B,NWK,MAXA,NNM,ESTFB,NSTF,NODES(1,NEL),NNODES,1)
C
  220 CONTINUE
      RETURN
      END
C
      SUBROUTINE ASSMBM(MAXA,B,EMSS,ESTFB,EMS1,XGRID,XNODE,UNODE,QNODE,
     1                  PSI,XG,WG,NODE,NODES,NELEM,NNODE,NNODES,NGQ,NNM,
     2                  NWK,NGRID,NSTF,IOUT)
C . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
C .                                                                   .
C .   P R O G R A M                                                   .
C .                  TO CALCULATE AND ASSEMBLE THE MASS ELEMENT       .
C .                  MATRIX INTO THE GLOBAL MASS MATRICES             .
C .                                                                   .
C . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION MAXA(NNM),B(NWK),EMSS(NNODES,NNODES),ESTFB(NSTF),
     1          EMS1(NNODE,NNODE),XGRID(NGRID),XNODE(NNODE),
     2          UNODE(NNODE),QNODE(NNODE),PSI(NNODE,NGQ),XG(NGQ),
     3          WG(NGQ),NODE(NNODE,NELEM),NODES(NNODES,NELEM)
      DATA DZERO / 0.D0 /
C
      DO 15 IG = 1 , NGQ
C
          CALL SHAPEF(XG(IG),UNODE,QNODE,XNODE,NNODE)
C
          DO 14 IN = 1 , NNODE
             PSI (IN,IG) = UNODE(IN)
   14     CONTINUE
C
   15 CONTINUE
C
C      BEGIN BIG LOOP ON ELEMENTS
C
      DO 10 I = 1 , NWK
         B(I) = DZERO
   10 CONTINUE
C
      DO 220 NEL = 1 , NELEM
C
         DO 40 II = 1 , NNODES
         DO 40 JJ = 1 , NNODES
            EMSS(II,JJ) = DZERO
   40    CONTINUE
C
         DO 50 LL = 1 , NNODE
            NE           = NODE(LL,NEL)
            XNODE(LL)    = XGRID(NE)
   50    CONTINUE
C
C      CALCULATE ELEMENT MATRICES FOR DIAGONAL
C      ELEMENTS OF MASS MATRIX
C
        CALL EMASSD(XNODE,XG,WG,NGQ,NNODE,EMS1,PSI)
C
          DO 70 NI = 1  , NNODE
          DO 70 NJ = NI , NNODE
C
               EMSS(NI,NJ) = EMSS(NI,NJ) + EMS1(NI,NJ)
C
   70    CONTINUE
C
         NL  = 0
         DO 210 IL = 1  , NNODES
         DO 210 JL = IL , NNODES
            NL        = NL + 1
            ESTFB(NL) = EMSS(IL,JL)
C
  210    CONTINUE
C
C      ASSEMBLE ELEMENT MATRICES INTO GLOBAL
C      MASS MATRIX
C
         CALL ADDVEC(B,NWK,MAXA,NNM,ESTFB,NSTF,NODES(1,NEL),NNODES,1)
C
  220 CONTINUE
      RETURN
      END
C
      SUBROUTINE EMASSD(XNODE,XG,WG,NGQ,NNODE,EMSS,PSI)
C . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
C .                                                                   .
C .   P R O G R A M                                                   .
C .                  TO CALCULATE AN ELEMENT MASS MATRIX              .
C .                                                                   .
C . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION XNODE(NNODE),EMSS(NNODE,NNODE),XG(NGQ),WG(NGQ),
     1          PSI(NNODE,NGQ)
      DATA DZERO / 0.D0 /, TWO / 2.D0 /
C
C      INITIALIZE ELEMENT ARRAY
C
      DO 10 I = 1 , NNODE
      DO 10 J = 1 , NNODE
         EMSS(I,J) = DZERO
   10 CONTINUE
C
C      FORM JACOBI DETERMINANT
C
      DETJ = (XNODE(NNODE) - XNODE(1)) / TWO
C
C      BEGIN INTEGRATION POINT LOOP
C
      DO 50 IG = 1 , NGQ
C
        RG = DETJ * XG(IG) + (XNODE(NNODE) + XNODE(1)) / TWO
C
        DO 40 I = 1 , NNODE
        DO 40 J = I , NNODE
C
          EMSS(I,J) = EMSS(I,J) + PSI(I,IG) * PSI(J,IG) * WG(IG) * DETJ
          EMSS(J,I) = EMSS(I,J)
C
   40  CONTINUE
C
   50 CONTINUE
      RETURN
      END
C
      SUBROUTINE ESTIFD(XNODE,XG,WG,NGQ,NNODE,EST,EMT,PSI,DPSI,
     1                  TT,TAU,ITORDR)
C . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
C .                                                                   .
C .   P R O G R A M                                                   .
C .                  TO CALCULATE AN ELEMENT AA AND SS MATRICES       .
C .                  FOR DIAGONAL PART OF POTENTIAL MATRICES          .
C .                                                                   .
C . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION XNODE(NNODE),EST(NNODE,NNODE),EMT(NNODE,NNODE),XG(NGQ),
     1          WG(NGQ),PSI(NNODE,NGQ),DPSI(NNODE,NGQ)
      DATA DZERO / 0.D0 /, TWO / 2.D0 /
C
C      INITIALIZE ELEMENT ARRAY
C
      DO 10 I = 1 , NNODE
      DO 10 J = 1 , NNODE
C
         EST(I,J) = DZERO
         EMT(I,J) = DZERO
C
   10 CONTINUE
C
C      FORM JACOBI DETERMINANT
C
      DETJ = (XNODE(NNODE) - XNODE(1)) / TWO
C
C      BEGIN INTEGRATION POINT LOOP
C
      DO 50 IG = 1 , NGQ
C
        RG = DETJ * XG(IG) + (XNODE(NNODE) + XNODE(1)) / TWO
        CALL POTCAL(RG,TT,TAU,DA,HH,DS,SS,ITORDR)
C
        DO 40 I = 1 , NNODE
        DO 40 J = I , NNODE
C
         EST(I,J) = EST(I,J) + (PSI(I,IG) * PSI(J,IG) * HH
     1            + DA * DPSI(I,IG) * DPSI(J,IG) / DETJ / DETJ) 
     2            * WG(IG) * DETJ
         EST(J,I) = EST(I,J)
C
         EMT(I,J) = EMT(I,J) + (PSI(I,IG) * PSI(J,IG) * SS
     1            + DS * DPSI(I,IG) * DPSI(J,IG) / DETJ / DETJ) 
     2            * WG(IG) * DETJ
         EMT(J,I) = EMT(I,J)

C
   40   CONTINUE
C
   50 CONTINUE
      RETURN
      END
C
      SUBROUTINE ADDVEC(A,NWT,MAXA,NNM,STIFF,NSTF,NODE,NNODE,KOD)
C . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
C .                                                                   .
C .   P R O G R A M                                                   .
C .                  TO ASSEMBLE ELEMENT VECTOR STIFF INTO GLOBAL     .
C .                  VECTOR A USING A COMPACT STORAGE FORM            .
C .                                                                   .
C . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(NWT),MAXA(NNM),STIFF(NSTF),NODE(NNODE)
      NS = 0
      IF (KOD .NE. 1) NS = NWT / 2
      NDF = 0
      DO 20 I = 1 , NNODE
         NI = NODE(I)
         IF (NI .GT. 0) THEN
            MNI = MAXA(NI)
            MI  = I
            DO 10 J = 1 , NNODE
               NJ = NODE(J)
               IF (NJ .GT. 0) THEN
                  NIJ = NI - NJ
                  IF (NIJ .GE. 0) THEN
                     MIJ  = MNI + NIJ
                     MNIJ = MI
                     IF (J .GE. I) MNIJ = J + NDF
                     A(MIJ + NS) = A(MIJ + NS) + STIFF(MNIJ)
                  END IF
               END IF
               MI = MI + NNODE - J
   10       CONTINUE
         END IF
         NDF = NDF + NNODE - I
   20 CONTINUE
      RETURN
      END
C
      SUBROUTINE BOUNDC(NODE,NODES,MAXA,MHT,NHT,IBOUND,NNODE,NNODES,
     1                  NELEM,MDIM,NN,NNE,IPRINT,IOUT)
C . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
C .                                                                   .
C .   P R O G R A M                                                   .
C .                  TO SET THE DIRICHLET AND NEUMANN BOUNDARY        .
C .                  CONDITIONS                                       .
C .                                                                   .
C . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION MAXA(1),MHT(1),NHT(1),NODE(NNODE,NELEM),
     1          NODES(NNODES,NELEM)
C
C      ESTABLISH BOUNDARY CONDITIONS
C
      GO TO (10,40,60,80),IBOUND
C
C      DIRICHLET - DIRICHLET BOUNDARY CONDITIONS
C
   10 IK    = 0
      NEL   = 1
      DO 20 IL = 1 , MDIM
         IK      = IK + 1
         NHT(IK) = NODES(IL,NEL)
   20 CONTINUE
      NEL  = NELEM
      DO 30 IL = 1 , MDIM
         IK       = IK + 1
         ILL      = MDIM * (NNODE - 1)
         NHT(IK)  = NODES(IL + ILL,NEL)
   30 CONTINUE
      GO TO 200
C
C      DIRICHLET - NEUMANN BOUNDARY CONDITIONS
C
   40 IK   = 0
      NEL  = 1
      DO 50 IL = 1 , MDIM
         IK       = IK + 1
         NHT(IK)  = NODES(IL,NEL)
   50 CONTINUE
      GO TO 200
C
C      NEUMANN - DIRICHLET BOUNDARY CONDITIONS
C
   60 IK    = 0
      NEL   = NELEM
      DO 70 IL = 1 , MDIM
         IK       = IK + 1
         ILL      = MDIM * (NNODE - 1)
         NHT(IK)  = NODES(IL + ILL,NEL)
   70 CONTINUE
      GO TO 200
C
C      NEUMANN - NEUMANN BOUNDARY CONDITION
C
   80 GO TO 300
C
  200 CONTINUE
C
C      ELIMINATE CORRESPONDING COLUMNS AND ROWS FOR THE
C      IMPLEMENTATION OF THE DIRICHLET BOUNDARY CONDITIONS
C      AND SET A NEW NUMERATION OF NODES
C
      DO 210 IL = 1 , NNE
         MAXA(IL) = IL
         MHT (IL) = 0
  210 CONTINUE
C
      DO 220 I = 1 , IK
         J        = NHT(I)
         MHT(J)   = 1
  220 CONTINUE
C
      DO 230 NEL = 1 , NELEM
      DO 230   I = 1 , NNODES
         K  = NODES(I,NEL)
         IF (K .EQ. 0) GO TO 230 !!! ADDING CHUKA FOR COMPAQ FORTRAN
         IF (MHT(K) .NE. 0) NODES(I,NEL) = 0
  230 CONTINUE
C
      DO 240 I = 1 , IK
         J       = NHT(I)
         MAXA(J) = 0
  240 CONTINUE
C
      NN  = 0
      DO 250 I = 1 , NNE
         MHT(I)   = 0
         IF (MAXA(I) .NE. 0) THEN
             NN       = NN + 1
             NHT(NN)  = I
             MHT(I)   = NN
         END IF
  250 CONTINUE
C
      DO 270 NEL = 1 , NELEM
         DO 260 I = 1 , NNODES
            K   = NODES(I,NEL)
            IF (K .NE. 0) THEN
               NODES(I,NEL) = MHT(K)
            END IF
  260    CONTINUE
  270 CONTINUE
C
  300 CONTINUE
C
      IF (IPRINT .GE. 3) THEN
         WRITE(IOUT,1020) NNODE,NELEM
         DO 320 NEL = 1 , NELEM
  320    WRITE(IOUT,1030) (NODES(IP,NEL),IP=1,NNODES)
         WRITE(IOUT,1040)
      END IF
      RETURN
 1020 FORMAT(//3X,'NODAL ARRAY NODES (',I3,',',I3,
     1       ') MODIFY ACCORDING TO DIRICHLET CONDITION :'/)
 1030 FORMAT(12I6)
 1040 FORMAT(/)
      END
C
      SUBROUTINE COLMHT(MHT,NNM,NNODE,NODE)
C . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
C .                                                                   .
C .   P R O G R A M                                                   .
C .                  TO CALCULATE COLUMN HEIGHTS                      .
C .                                                                   .
C . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION MHT(NNM),NODE(NNODE)
      NMAX = 1000000
      DO 10 I = 1 , NNODE
         NDI = NODE(I)
         IF (NDI .NE. 0) THEN
            IF ((NDI - NMAX) .LT. 0) NMAX = NDI
         END IF
   10 CONTINUE
      DO 20 I = 1 , NNODE
         NDI = NODE(I)
         IF (NDI .NE. 0) THEN
            NDIF = NDI - NMAX
            IF (NDIF .GT. MHT(NDI)) MHT(NDI) = NDIF
         END IF
   20 CONTINUE
      RETURN
      END
C
      SUBROUTINE ERRDIM(N,I,IOUT)
C . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
C .                                                                   .
C .   P R O G R A M                                                   .
C .                  TO PRINT MESSAGES WHEN HIGH-SPEED STORAGE        .
C .                  IS EXCEEDED                                      .
C .                                                                   .
C . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
      GO TO (1,2,3),I
    1 WRITE(IOUT,1010)
      WRITE(IOUT,1100) N
      STOP
    2 WRITE(IOUT,1020)
      WRITE(IOUT,1110) N
      STOP
    3 WRITE(IOUT,1030)
      WRITE(IOUT,1140) N
      STOP
      RETURN
 1010 FORMAT(//3X,'*****   ERRDIM  ***** ',
     *       //3X,'NOT ENOUGH STORAGE FOR NODAL POINT DATA'/)
 1020 FORMAT(//3X,'*****   ERRDIM  ***** ',
     *       //3X,'NOT ENOUGH STORAGE FOR ASSEMBLAGE OF GLOBAL ',
     *            'STIFFNESS AND MASS MATRICES '/)
 1030 FORMAT(//3X,'*****   ERRDIM  ***** ',
     *       //3X,'NOT ENOUGH STORAGE FOR TIME',
     *            ' DEPENDENT PROBLEM SOLUTION PHASE'/)
 1100 FORMAT(/3X,'STORAGE OF ARRAY ITOT IS EXCEEDED BY ',I9/)
 1110 FORMAT(/3X,'STORAGE OF ARRAY TOT IS EXCEEDED BY ',I9/)
 1140 FORMAT(/3X,'STORAGE OF ARRAY ZTOT IS EXCEEDED BY ',I9/)
      END
C
      SUBROUTINE FEGRID(RMESH,NMESH,RGRID,NGRID,NELEM,NPOL)
C . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
C .                                                                   .
C .   P R O G R A M                                                   .
C .                  TO CALCULATE NODAL POINTS RGRID(NGRID)           .
C .                                                                   .
C . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION RMESH(NMESH),RGRID(1)
      NG = 0
      NL = 0
      DO 20 I = 3 , NMESH, 2
          XM  = RMESH(I) - RMESH(I-2)
          XS  = XM / IDINT(RMESH(I-1)) / DFLOAT(NPOL)
          NM  = IDINT(XM / XS + 0.000000000001)
          NL  = NL + IDINT(RMESH(I-1))
          DO 10 J = 1 , NM
              NG            = NG + 1
              RGRID(NG)     = RMESH(I-2) + (J-1) * XS
   10     CONTINUE
   20 CONTINUE
      NELEM        = NL
      NGRID        = NG + 1
      RGRID(NGRID) = RMESH(NMESH)
      RETURN
      END
C
      SUBROUTINE GAULEG(X1,X2,X,W,N,EPS)
C . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
C .                                                                   .
C .   P R O G R A M                                                   .
C .                  TO CALCULATE THE NODES AND WEIGHTS OF THE        .
C .                  GAUSS-LEGENDRE QUADRATURE. PUBLISHED IN:         .
C .                  W.H. PRESS, B.F. FLANERY, S.A. TEUKOLSKY,        .
C .                  AND W.T.VETTERLEY, NUMERICAL RECIPES: THE        .
C .                  ART OF SCIENTIFIC COMPUTING (CAMBRIDGE           .
C .                  UNIVERSITY PRESS, CAMBRIDGE, 1986), CHAP. 4      .
C .                                                                   .
C . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION X(N),W(N)
      DATA DZERO/0.D0/, QUAT/0.25D0/, HALF/0.5D0/, ONE/1.D0/, TWO/2.D0/
      PI = DACOS(-ONE)
      M=(N+1)/2
      XM=HALF*(X2+X1)
      XL=HALF*(X2-X1)
      DO 12 I = 1 , M
        Z=DCOS(PI*(I-QUAT)/(N+HALF))
    1   CONTINUE
          P1=ONE
          P2=DZERO
          DO 11 J = 1 , N
            P3=P2
            P2=P1
            P1=((TWO*J-ONE)*Z*P2-(J-ONE)*P3)/J
   11     CONTINUE
          PP=N*(Z*P1-P2)/(Z*Z-ONE)
          Z1=Z
          Z=Z1-P1/PP
        IF (ABS(Z-Z1).GT.EPS) GO TO 1
        X(I)=XM-XL*Z
        X(N+1-I)=XM+XL*Z
        W(I)=TWO*XL/((ONE-Z*Z)*PP*PP)
        W(N+1-I)=W(I)
   12 CONTINUE
      RETURN
      END
C
      SUBROUTINE MAXHT(MAXA,MHT,NODES,NNODES,NELEM,NN,NNM,NWK,NWM,
     1                 MK,IPRINT,IOUT)
C . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
C .                                                                   .
C .   P R O G R A M                                                   .
C .                  TO CALCULATE ADDRESSES OF DIAGONAL ELEMENTS      .
C .                  IN BANDED MATRIX                                 .
C .                                                                   .
C . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION MAXA(NNM),MHT(NNM),NODES(NNODES,NELEM)
C
C      CALCULATE COLUMN HEIGHTS
C
      DO 10 IL = 1 , NN
         MHT (IL) = 0
         MAXA(IL) = 0
   10 CONTINUE
C
      DO 20 NEL = 1 , NELEM
         CALL COLMHT(MHT,NNM,NNODES,NODES(1,NEL))
   20 CONTINUE
C
C      SET ADDRESSES OF DIAGONAL ELEMENTS IN MAXA
C
      MAXA(1) = 1
      MAXA(2) = 2
      MK      = 0
      DO 30 IL = 1 , NN
         IF (MHT(IL) .GT. MK) MK = MHT(IL)
         MAXA(IL+1) = MAXA(IL) + MHT(IL) + 1
   30 CONTINUE
C
      MK   = MK + 1
      NNM  = NN + 1
      NWK  = MAXA(NNM) - 1
      NWM  = NWK
C
      IF (IPRINT .GE. 3) THEN
         WRITE(IOUT,1010) NNM
         WRITE(IOUT,1020) (MAXA(I),I=1,NNM)
         WRITE(IOUT,1030)
      END IF
C
      IF (IPRINT .GE. 3) THEN
 1010 FORMAT(/3X,'ADDRESSES OF DIAGONAL ELEMENTS SET IN PROFILE ',
     * 'MATRIX MAXA(',I5,') :'/)
 1020 FORMAT(10I7)
 1030 FORMAT(/)
      END IF
      RETURN
      END
C
      SUBROUTINE NODGEN(NODE,NNODE,NODES,NNODES,MDIM,NELEM,IPRINT,IOUT)
C . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
C .                                                                   .
C .   P R O G R A M                                                   .
C .                  TO CALCULATE THE NODAL POINT DISTRIBUTION        .
C .                                                                   .
C . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION NODE(NNODE,NELEM),NODES(NNODES,NELEM)
C
C      NUMERATE THE NODES
C
      DO I = 1 , NNODES
        DO J = 1 , NELEM
          NODES(I,J) = 0
          IF (I .LE. NNODE) NODE(I,J) = 0
        END DO
      END DO
C
      DO 10 NI = 1 , NNODE
         NODE(NI,1) = NI
   10 CONTINUE
C
      DO 30 NEL = 2 , NELEM
         DO 20 ND = 1 , NNODE
            NODE(ND,NEL) = NODE(NNODE,NEL-1) + ND - 1
   20    CONTINUE
   30 CONTINUE
C
C      GENERATE NODAL NUMBERS FOR MULTICHANNEL CASE
C
      DO 50 NEL = 1 , NELEM
      DO 50 NL  = 1 , NNODE
         DO 40 KL = 1 , MDIM
            IL = (NL - 1) * MDIM + KL
            JL = NODE(NL,NEL) * MDIM + KL - MDIM
            NODES(IL,NEL) = JL
   40    CONTINUE
   50 CONTINUE
C
      IF (IPRINT .GE. 3) THEN
         WRITE(IOUT,1000)
         WRITE(IOUT,1010) NNODE,NELEM
         DO 160 NEL = 1 , NELEM
  160    WRITE(IOUT,1020) NEL,(NODE(IP,NEL),IP=1,NNODE)
         WRITE(IOUT,1030) NNODE,NELEM
         DO 170 NEL = 1 , NELEM
  170    WRITE(IOUT,1040) (NODES(IP,NEL),IP=1,NNODES)
         WRITE(IOUT,1050)
      END IF
      RETURN
 1000 FORMAT(//15X,'N O D A L   D A T A   I N F O R M A T I O N '/
     *         15X,'------------------------------------------- '/)
 1010 FORMAT(/20X,'NODAL ARRAY NODE (',I2,' , ',I4,') :'/)
 1020 FORMAT(5X,'ELEMENT NO ',I5,5X,10I5)
 1030 FORMAT(//20X,'NODAL ARRAY NODES (',I4,' , ',I4,') :'/)
 1040 FORMAT(12I6)
 1050 FORMAT(/)
      END
C
      SUBROUTINE SHAPEF(X,PSI,DPSI,XNODE,NPOL1)
C . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
C .                                                                   .
C .   P R O G R A M                                                   .
C .                  TO CALCULATE SHAPE FUNCTIONS PSI(I) AND THEIR    .
C .                  DERIVATIVES DPSI(I) WITH RESPECT TO THE MASTER   .
C .                  ELEMENT COORDINATE AT A SPECIFIED VALUE OF X     .
C .                                                                   .
C . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION PSI(NPOL1),DPSI(NPOL1),XNODE(NPOL1)
      DATA DZERO / 0.D0 /, ONE / 1.D0 /, TWO / 2.D0 /
      NPOL  = NPOL1 - 1
      HNODE = TWO / DFLOAT(NPOL)
      DO 10 IP = 1 , NPOL1
          XNODE(IP) = - ONE + HNODE * (IP - 1)
   10 CONTINUE
      DO 40 NP = 1 , NPOL1
          PSIN       = ONE
          PSI(NP)    = ONE
          DPSI(NP)   = DZERO
          DO 30 IP = 1 , NPOL1
              IF (IP .NE. NP) THEN
                  PSI(NP) = PSI(NP) * (X - XNODE(IP))
                  PSIN    = PSIN * (XNODE(NP) - XNODE(IP))
                  PSID    = ONE
                  DO 20 JP = 1 , NPOL1
                      IF (JP .NE. IP .AND. JP .NE. NP) THEN
                           PSID = PSID * (X - XNODE(JP))
                      END IF
   20             CONTINUE
                  DPSI(NP) = DPSI(NP) + PSID
              END IF
   30     CONTINUE
          PSI(NP)  = PSI(NP)  / PSIN
          DPSI(NP) = DPSI(NP) / PSIN
   40 CONTINUE
      RETURN
      END
C
      SUBROUTINE MULTC(ZTT,B,ZRR,MAXA,NN,NWM)
C . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
C
C     P R O G R A M                                                 .
C         TO EVALUATE PRODUCT OF B TIMES RR AND STORE RESULT IN TT  .
C
C . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
      IMPLICIT REAL*8 (A-H,O-Y)
      IMPLICIT COMPLEX*16 (Z)
      DIMENSION ZTT(1),B(1),ZRR(1),MAXA(1)
      IF (NWM .GT. NN) GO TO 20
      DO 10 I = 1 , NN
   10 ZTT(I) = B(I) * ZRR(I)
      RETURN
   20 DO 40 I = 1 , NN
   40 ZTT(I) = DCMPLX(0.D0,0.D0)
      DO 100 I = 1 , NN
        KL = MAXA(I)
        KU = MAXA(I+1)-1
        II = I + 1
        ZCC = ZRR(I)
        DO 100 KK = KL , KU
          II = II - 1
  100   ZTT(II) = ZTT(II) + B(KK) * ZCC
      IF (NN .EQ. 1) RETURN
      DO 200 I = 2 , NN
        KL = MAXA(I) + 1
        KU = MAXA(I+1) - 1
        IF (KU - KL) 200,210,210
  210   II = I
        ZAA = DCMPLX(0.D0,0.D0)
        DO 220 KK = KL , KU
          II = II - 1
  220   ZAA = ZAA + B(KK) * ZRR(II)
        ZTT(I) = ZTT(I) + ZAA
  200 CONTINUE
      RETURN
      END
C
      SUBROUTINE MULTCC(ZTT,ZB,ZRR,MAXA,NN,NWM)
C . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
C
C     P R O G R A M                                                 .
C         TO EVALUATE PRODUCT OF B TIMES RR AND STORE RESULT IN TT  .
C
C . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
      IMPLICIT REAL*8(A-H,O-Y)
      IMPLICIT COMPLEX*16 (Z)
      DIMENSION ZTT(1),ZB(1),ZRR(1),MAXA(1)
      IF (NWM .GT. NN) GO TO 20
      DO 10 I = 1 , NN
   10 ZTT(I) = ZB(I) * ZRR(I)
      RETURN
   20 DO 40 I = 1 , NN
   40 ZTT(I) = DCMPLX(0.D0,0.D0)
      DO 100 I = 1 , NN
        KL = MAXA(I)
        KU = MAXA(I+1) - 1
        II = I + 1
        ZCC = ZRR(I)
        DO 100 KK = KL , KU
          II = II - 1
  100   ZTT(II) = ZTT(II) + ZB(KK) * ZCC
      IF (NN .EQ. 1) RETURN
      DO 200 I = 2 , NN
        KL = MAXA(I) + 1
        KU = MAXA(I+1) - 1
        IF (KU - KL) 200,210,210
  210   II = I
        ZAA = DCMPLX(0.D0,0.D0)
        DO 220 KK = KL , KU
          II = II - 1
  220   ZAA = ZAA + ZB(KK) * ZRR(II)
        ZTT(I) = ZTT(I) + ZAA
  200 CONTINUE
      RETURN
      END
C
      SUBROUTINE REDBAC(ZA,ZV,MAXA,NN)
C . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
C .                                                                 .
C .   P R O G R A M                                                 .
C .        TO REDUCE AND BACK-SUBSTITUTE ITERATION VECTORS          .
C .                                                                 .
C . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
      IMPLICIT REAL*8 (A-H,O-Y)
      IMPLICIT COMPLEX*16 (Z)
      DIMENSION ZA(1),ZV(1),MAXA(1)
      DO 400 N = 1 , NN
        KL = MAXA(N) + 1
        KU = MAXA(N+1) - 1
        IF (KU - KL) 400,410,410
  410   K = N
        ZC = DCMPLX(0.D0,0.D0)
        DO 420 KK = KL , KU
          K = K - 1
  420   ZC = ZC + ZA(KK) * ZV(K)
        ZV(N) = ZV(N) - ZC
  400 CONTINUE
      DO 480 N = 1 , NN
        K = MAXA(N)
  480 ZV(N) = ZV(N) / ZA(K)
      IF (NN .EQ. 1) RETURN
      N = NN
      DO 500 L = 2 , NN
        KL = MAXA(N) + 1
        KU = MAXA(N+1) - 1
        IF (KU - KL) 500,510,510
  510   K = N
        DO 520 KK = KL , KU
          K = K - 1
  520   ZV(K) = ZV(K) - ZA(KK) * ZV(N)
  500 N = N - 1
      RETURN
      END
C
      SUBROUTINE DECOMC(ZA,MAXA,NN,ISH,IFPR,IOUT)
C . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
C .                                                                   .
C .   P R O G R A M                                                   .
C .         TO CALCULATE (L)*(D)*(L)(T) FACTORIZATION OF              .
C .         STIFFNESS MATRIX                                          .
C .                                                                   .
C . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
      IMPLICIT REAL*8 (A-H,O-Y)
      IMPLICIT COMPLEX*16 (Z)
      DIMENSION ZA(1),MAXA(1)
C
      IF (NN .EQ. 1) RETURN
      NLOW = 0
      DO 200 N = 1 , NN
        KN = MAXA(N)
        KL = KN+1
        KU = MAXA(N+1) - 1
        KH = KU - KL
        IF (KH) 200,240,210
  210   K = N - KH
        IC = 0
        KLT = KU
        DO 260 J = 1 , KH
          IC = IC + 1
          KLT = KLT - 1
          KI = MAXA(K)
          ND = MAXA(K+1) - KI - 1
          IF (ND) 260,260,270
  270     KK = MIN0(IC,ND)
          ZC = DCMPLX(0.D0,0.D0)
          DO 280 L = 1 , KK
  280     ZC = ZC + ZA(KI+L) * ZA(KLT+L)
          ZA(KLT) = ZA(KLT) - ZC
  260   K = K + 1
  240   K = N
        ZB = DCMPLX(0.D0,0.D0)
        DO 300 KK = KL , KU
          K = K - 1
          KI = MAXA(K)
          ZC = ZA(KK) / ZA(KI)
          IF (CDABS(ZC) .LT. 1.E12) GO TO 290
          WRITE (IOUT,2010) N,ZC
          STOP
  290     ZB = ZB + ZC * ZA(KK)
  300   ZA(KK) = ZC
        ZA(KN) = ZA(KN) - ZB
  200 CONTINUE
      RETURN
 2000 FORMAT(//48H STOP - STIFFNESS MATRIX NOT POSITIVE DEFINITE  ,//
     1         32H NONPOSITIVE PIVOT FOR EQUATION  ,I6,//
     2         10H PIVOT =  ,E20.12)
 2010 FORMAT (//47H STOP - STURM SEQUENSE CHECK FAILED BECAUSE OF    ,/
     18X,35HMULTIPLIER GROWTH FOR COLUMN NUMBER,I7,//
     212H MULTIPLIER=,2E20.8)
 2020 FORMAT(1X,'THERE ARE ',I4,' ROOTS LOWER THEN SHIFT')
      END